@@ -26,6 +26,10 @@
 #include <config.h>
 #endif
 
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -88,6 +92,12 @@ gchar *xfce_backdrop_choose_next         (XfceBackdrop *backdrop);
 gchar *xfce_backdrop_choose_random       (XfceBackdrop *backdrop);
 gchar *xfce_backdrop_choose_chronological(XfceBackdrop *backdrop);
 
+static void cb_xfce_backdrop_image_files_changed(GFileMonitor     *monitor,
+                                                 GFile            *file,
+                                                 GFile            *other_file,
+                                                 GFileMonitorEvent event,
+                                                 gpointer          user_data);
+
 struct _XfceBackdropPriv
 {
     gint width, height;
@@ -95,7 +105,6 @@ struct _XfceBackdropPriv
 
     GdkPixbuf *pix;
     XfceBackdropImageData *image_data;
-    gboolean cache_pixbuf;
 
     XfceBackdropColorStyle color_style;
     GdkColor color1;
@@ -233,7 +242,7 @@ create_gradient(GdkColor *color1, GdkColor *color2, gint width, gint height,
     return pix;
 }
 
-static void
+void
 xfce_backdrop_clear_cached_image(XfceBackdrop *backdrop)
 {
     g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
@@ -245,10 +254,25 @@ xfce_backdrop_clear_cached_image(XfceBackdrop *backdrop)
     backdrop->priv->pix = NULL;
 }
 
+static void
+xfdesktop_backdrop_clear_directory_monitor(XfceBackdrop *backdrop)
+{
+    g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
+
+    if(backdrop->priv->monitor) {
+            g_signal_handlers_disconnect_by_func(G_OBJECT(backdrop->priv->monitor),
+                                                 G_CALLBACK(cb_xfce_backdrop_image_files_changed),
+                                                 backdrop);
+
+            g_object_unref(backdrop->priv->monitor);
+            backdrop->priv->monitor = NULL;
+        }
+}
+
 /* we compare by the collate key so the image listing is the same as how
  * xfdesktop-settings displays the images */
 static gint
-compare_by_collate_key(const gchar *a, const gchar *b)
+glist_compare_by_collate_key(const gchar *a, const gchar *b)
 {
     gint ret;
     gchar *a_key = g_utf8_collate_key_for_filename(a, -1);
@@ -262,12 +286,26 @@ compare_by_collate_key(const gchar *a, const gchar *b)
     return ret;
 }
 
+typedef struct
+{
+    gchar *key;
+    gchar *string;
+} KeyStringPair;
+
+static int
+qsort_compare_pair_by_key(const void *a, const void *b)
+{
+    const gchar *a_key = ((const KeyStringPair *)a)->key;
+    const gchar *b_key = ((const KeyStringPair *)b)->key;
+    return g_strcmp0(a_key, b_key);
+}
+
 static void
-cb_xfce_backdrop__image_files_changed(GFileMonitor     *monitor,
-                                      GFile            *file,
-                                      GFile            *other_file,
-                                      GFileMonitorEvent event,
-                                      gpointer          user_data)
+cb_xfce_backdrop_image_files_changed(GFileMonitor     *monitor,
+                                     GFile            *file,
+                                     GFile            *other_file,
+                                     GFileMonitorEvent event,
+                                     gpointer          user_data)
 {
     XfceBackdrop *backdrop = XFCE_BACKDROP(user_data);
     gchar *changed_file = NULL;
@@ -275,8 +313,15 @@ cb_xfce_backdrop__image_files_changed(GFileMonitor     *monitor,
 
     switch(event) {
         case G_FILE_MONITOR_EVENT_CREATED:
+            if(!xfce_backdrop_get_cycle_backdrop(backdrop)) {
+                /* If we're not cycling, do nothing, it's faster :) */
+                break;
+            }
+
             changed_file = g_file_get_path(file);
 
+            XF_DEBUG("file added: %s", changed_file);
+
             /* Make sure we don't already have the new file in the list */
             if(g_list_find(backdrop->priv->image_files, changed_file)) {
                 g_free(changed_file);
@@ -290,16 +335,29 @@ cb_xfce_backdrop__image_files_changed(GFileMonitor     *monitor,
                 return;
             }
 
-            /* It is an image file and we don't have it in our list, add it
-             * sorted to our list, don't free changed file, that will happen
-             * when it is removed */
-            backdrop->priv->image_files = g_list_insert_sorted(backdrop->priv->image_files,
-                                                               changed_file,
-                                                               (GCompareFunc)compare_by_collate_key);
+            if(!xfce_backdrop_get_random_order(backdrop)) {
+                /* It is an image file and we don't have it in our list, add
+                 * it sorted to our list, don't free changed file, that will
+                 * happen when it is removed */
+                backdrop->priv->image_files = g_list_insert_sorted(backdrop->priv->image_files,
+                                                                   changed_file,
+                                                                   (GCompareFunc)glist_compare_by_collate_key);
+            } else {
+                /* Same as above except we don't care about the list's order
+                 * so just add it */
+                backdrop->priv->image_files = g_list_prepend(backdrop->priv->image_files, changed_file);
+            }
             break;
         case G_FILE_MONITOR_EVENT_DELETED:
+            if(!xfce_backdrop_get_cycle_backdrop(backdrop)) {
+                /* If we're not cycling, do nothing, it's faster :) */
+                break;
+            }
+
             changed_file = g_file_get_path(file);
 
+            XF_DEBUG("file deleted: %s", changed_file);
+
             /* find the file in the list */
             item = g_list_find_custom(backdrop->priv->image_files,
                                       changed_file,
@@ -309,6 +367,23 @@ cb_xfce_backdrop__image_files_changed(GFileMonitor     *monitor,
             if(item)
                 backdrop->priv->image_files = g_list_delete_link(backdrop->priv->image_files, item);
 
+            g_free(changed_file);
+            break;
+        case G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT:
+            changed_file = g_file_get_path(file);
+
+            XF_DEBUG("file changed: %s", changed_file);
+            XF_DEBUG("image_path: %s", backdrop->priv->image_path);
+
+            if(g_strcmp0(changed_file, backdrop->priv->image_path) == 0) {
+                DBG("match");
+                /* clear the outdated backdrop */
+                xfce_backdrop_clear_cached_image(backdrop);
+
+                /* backdrop changed! */
+                g_signal_emit(G_OBJECT(backdrop), backdrop_signals[BACKDROP_CHANGED], 0);
+            }
+
             g_free(changed_file);
             break;
         default:
@@ -316,14 +391,93 @@ cb_xfce_backdrop__image_files_changed(GFileMonitor     *monitor,
     }
 }
 
+/* Equivalent to (except for not being a stable sort), but faster than
+ * g_list_sort(list, (GCompareFunc)glist_compare_by_collate_key) */
+static GList *
+sort_image_list(GList *list, guint list_size)
+{
+    KeyStringPair *array;
+    guint i;
+    GList *l;
+
+    TRACE("entering");
+
+    g_assert(g_list_length(list) == list_size);
+
+#define TEST_IMAGE_SORTING 0
+
+#if TEST_IMAGE_SORTING
+    GList *list2 = g_list_copy(list);
+#endif
+
+    /* Allocate an array of the same size as list */
+    array = g_malloc(list_size * sizeof(array[0]));
+
+    /* Copy list contents to the array and generate collation keys */
+    for(l = list, i = 0; l; l = l->next, ++i) {
+        array[i].string = l->data;
+        array[i].key = g_utf8_collate_key_for_filename(array[i].string, -1);
+    }
+
+    /* Sort the array */
+    qsort(array, list_size, sizeof(array[0]), qsort_compare_pair_by_key);
+
+    /* Copy sorted array back to the list and deallocate the collation keys */
+    for(l = list, i = 0; l; l = l->next, ++i) {
+        l->data = array[i].string;
+        g_free(array[i].key);
+    }
+
+    g_free(array);
+
+#if TEST_IMAGE_SORTING
+    list2 = g_list_sort(list2, (GCompareFunc)glist_compare_by_collate_key);
+    if(g_list_length(list) != g_list_length(list2)) {
+        printf("Image sorting test FAILED: list size is not correct.");
+    } else {
+        GList *l2;
+        gboolean data_matches = TRUE, pointers_match = TRUE;
+        for(l = list, l2 = list2; l; l = l->next, l2 = l2->next) {
+            if(g_strcmp0(l->data, l2->data) != 0)
+                data_matches = FALSE;
+            if(l->data != l2->data)
+                pointers_match = FALSE;
+        }
+        if(data_matches) {
+            printf("Image sorting test SUCCEEDED: ");
+            if(pointers_match) {
+                printf("both data and pointers are correct.");
+            } else {
+                printf("data matches but pointers do not match. "
+                        "This is caused by unstable sorting.");
+            }
+        }
+        else {
+            printf("Image sorting test FAILED: ");
+            if(pointers_match) {
+                printf("data does not match but pointers do match. "
+                        "Something went really wrong!");
+            }
+            else {
+                printf("neither data nor pointers match.");
+            }
+        }
+    }
+    putchar('\n');
+#endif
+
+    return list;
+}
+
 /* Returns a GList of all the image files in the parent directory of filename */
 static GList *
-list_image_files_in_dir(const gchar *filename)
+list_image_files_in_dir(XfceBackdrop *backdrop, const gchar *filename)
 {
     GDir *dir;
     gboolean needs_slash = TRUE;
     const gchar *file;
     GList *files = NULL;
+    guint file_count = 0;
     gchar *dir_name;
 
     dir_name = g_path_get_dirname(filename);
@@ -340,41 +494,50 @@ list_image_files_in_dir(const gchar *filename)
     while((file = g_dir_read_name(dir))) {
         gchar *current_file = g_strdup_printf(needs_slash ? "%s/%s" : "%s%s",
                                               dir_name, file);
-        if(xfdesktop_image_file_is_valid(current_file))
-            files = g_list_insert_sorted(files, current_file, (GCompareFunc)compare_by_collate_key);
-        else
+        if(xfdesktop_image_file_is_valid(current_file)) {
+            files = g_list_prepend(files, current_file);
+            ++file_count;
+        } else
             g_free(current_file);
     }
 
     g_dir_close(dir);
     g_free(dir_name);
 
+    /* Only sort if there's more than 1 item and we're not randomly picking
+     * images from the list */
+    if(file_count > 1 && !xfce_backdrop_get_random_order(backdrop)) {
+        files = sort_image_list(files, file_count);
+    }
+
     return files;
 }
 
 static void
 xfce_backdrop_load_image_files(XfceBackdrop *backdrop)
 {
-    /* generate the image_files list if it doesn't exist and monitor that
-     * directory so we can update the list */
-    if(backdrop->priv->image_files == NULL && backdrop->priv->image_path) {
+    TRACE("entering");
+
+    /* generate the image_files list if it doesn't exist and we're cycling
+     * backdrops */
+    if(backdrop->priv->image_files == NULL &&
+       backdrop->priv->image_path &&
+       xfce_backdrop_get_cycle_backdrop(backdrop)) {
+        backdrop->priv->image_files = list_image_files_in_dir(backdrop, backdrop->priv->image_path);
+
+        xfdesktop_backdrop_clear_directory_monitor(backdrop);
+    }
+
+    /* Always monitor the directory even if we aren't cycling so we know if
+     * our current wallpaper has changed by an external program/script */
+    if(backdrop->priv->image_path && !backdrop->priv->monitor) {
         gchar *dir_name = g_path_get_dirname(backdrop->priv->image_path);
         GFile *gfile = g_file_new_for_path(dir_name);
 
-        backdrop->priv->image_files = list_image_files_in_dir(backdrop->priv->image_path);
-
-        if(backdrop->priv->monitor) {
-            g_signal_handlers_disconnect_by_func(G_OBJECT(backdrop->priv->monitor),
-                                                 G_CALLBACK(cb_xfce_backdrop__image_files_changed),
-                                                 backdrop);
-
-            g_object_unref(backdrop->priv->monitor);
-        }
-
-        /* monitor the new directory for changes */
+        /* monitor the directory for changes */
         backdrop->priv->monitor = g_file_monitor(gfile, G_FILE_MONITOR_NONE, NULL, NULL);
         g_signal_connect(backdrop->priv->monitor, "changed",
-                         G_CALLBACK(cb_xfce_backdrop__image_files_changed),
+                         G_CALLBACK(cb_xfce_backdrop_image_files_changed),
                          backdrop);
 
         g_free(dir_name);
@@ -396,9 +559,6 @@ xfce_backdrop_choose_next(XfceBackdrop *backdrop)
 
     filename = backdrop->priv->image_path;
 
-    if(backdrop->priv->image_files == NULL)
-        xfce_backdrop_load_image_files(backdrop);
-
     if(!backdrop->priv->image_files)
         return NULL;
 
@@ -433,8 +593,6 @@ xfce_backdrop_choose_random(XfceBackdrop *backdrop)
 
     g_return_val_if_fail(XFCE_IS_BACKDROP(backdrop), NULL);
 
-    xfce_backdrop_load_image_files(backdrop);
-
     if(!backdrop->priv->image_files)
         return NULL;
 
@@ -471,8 +629,6 @@ xfce_backdrop_choose_chronological(XfceBackdrop *backdrop)
 
     g_return_val_if_fail(XFCE_IS_BACKDROP(backdrop), NULL);
 
-    xfce_backdrop_load_image_files(backdrop);
-
     if(!backdrop->priv->image_files)
         return NULL;
 
@@ -488,7 +644,7 @@ xfce_backdrop_choose_chronological(XfceBackdrop *backdrop)
     /* Figure out which image to display based on what time of day it is
      * and how many images we have to work with */
     epoch = (gdouble)g_date_time_get_hour(datetime) / (24.0f / MIN(n_items, 24.0f));
-    DBG("epoch %d, hour %d, items %d", epoch, g_date_time_get_hour(datetime), n_items);
+    XF_DEBUG("epoch %d, hour %d, items %d", epoch, g_date_time_get_hour(datetime), n_items);
 
     new_file = g_list_nth(backdrop->priv->image_files, epoch);
 
@@ -642,14 +798,7 @@ xfce_backdrop_finalize(GObject *object)
 
     xfce_backdrop_clear_cached_image(backdrop);
 
-    if(backdrop->priv->monitor) {
-        g_signal_handlers_disconnect_by_func(G_OBJECT(backdrop->priv->monitor),
-                                             G_CALLBACK(cb_xfce_backdrop__image_files_changed),
-                                             backdrop);
-
-        g_object_unref(backdrop->priv->monitor);
-        backdrop->priv->monitor = NULL;
-    }
+    xfdesktop_backdrop_clear_directory_monitor(backdrop);
 
     /* Free the image files list */
     if(backdrop->priv->image_files) {
@@ -1003,26 +1152,22 @@ xfce_backdrop_set_image_filename(XfceBackdrop *backdrop, const gchar *filename)
         return;
 
     /* We need to free the image_files if image_path changed directories */
-    if(backdrop->priv->image_files) {
+    if(backdrop->priv->image_files || backdrop->priv->monitor) {
         if(backdrop->priv->image_path)
             old_dir = g_path_get_dirname(backdrop->priv->image_path);
         if(filename)
             new_dir = g_path_get_dirname(filename);
 
-        /* Directories did change, free list */
+        /* Directories did change */
         if(g_strcmp0(old_dir, new_dir) != 0) {
-            g_list_free_full(backdrop->priv->image_files, g_free);
-            backdrop->priv->image_files = NULL;
+            /* Free the image list if we had one */
+            if(backdrop->priv->image_files) {
+                g_list_free_full(backdrop->priv->image_files, g_free);
+                backdrop->priv->image_files = NULL;
+            }
 
             /* release the directory monitor */
-            if(backdrop->priv->monitor) {
-                g_signal_handlers_disconnect_by_func(G_OBJECT(backdrop->priv->monitor),
-                                                     G_CALLBACK(cb_xfce_backdrop__image_files_changed),
-                                                     backdrop);
-
-                g_object_unref(backdrop->priv->monitor);
-                backdrop->priv->monitor = NULL;
-            }
+            xfdesktop_backdrop_clear_directory_monitor(backdrop);
         }
 
         g_free(old_dir);
@@ -1039,6 +1184,8 @@ xfce_backdrop_set_image_filename(XfceBackdrop *backdrop, const gchar *filename)
 
     xfce_backdrop_clear_cached_image(backdrop);
 
+    xfce_backdrop_load_image_files(backdrop);
+
     g_signal_emit(G_OBJECT(backdrop), backdrop_signals[BACKDROP_CHANGED], 0);
 }
 
@@ -1148,7 +1295,7 @@ xfce_backdrop_timer(XfceBackdrop *backdrop)
         /* remove old timer first */
         xfce_backdrop_remove_backdrop_timer(backdrop);
 
-        DBG("calling g_timeout_add_seconds, interval is %d", cycle_interval);
+        XF_DEBUG("calling g_timeout_add_seconds, interval is %d", cycle_interval);
         backdrop->priv->cycle_timer_id = g_timeout_add_seconds(cycle_interval,
                                                                (GSourceFunc)xfce_backdrop_timer,
                                                                backdrop);
@@ -1240,7 +1387,7 @@ xfce_backdrop_set_cycle_timer(XfceBackdrop *backdrop, guint cycle_timer)
             }
 
         if(cycle_interval != 0) {
-            DBG("calling g_timeout_add_seconds, interval is %d", cycle_interval);
+            XF_DEBUG("calling g_timeout_add_seconds, interval is %d", cycle_interval);
             backdrop->priv->cycle_timer_id = g_timeout_add_seconds(cycle_interval,
                                                                    (GSourceFunc)xfce_backdrop_timer,
                                                                    backdrop);
@@ -1279,12 +1426,17 @@ xfce_backdrop_set_cycle_backdrop(XfceBackdrop *backdrop,
         /* Start or stop the backdrop changing */
         xfce_backdrop_set_cycle_timer(backdrop,
                                       xfce_backdrop_get_cycle_timer(backdrop));
-    }
 
-    /* If we're not cycling anymore, free the image files list */
-    if(!backdrop->priv->cycle_backdrop && backdrop->priv->image_files) {
-        g_list_free_full(backdrop->priv->image_files, g_free);
-        backdrop->priv->image_files = NULL;
+        if(cycle_backdrop) {
+            /* We're cycling now, so load up an image list */
+            xfce_backdrop_load_image_files(backdrop);
+        }
+        else if(backdrop->priv->image_files)
+        {
+            /* we're not cycling anymore, free the image files list */
+            g_list_free_full(backdrop->priv->image_files, g_free);
+            backdrop->priv->image_files = NULL;
+        }
     }
 }
 
@@ -1348,7 +1500,17 @@ xfce_backdrop_set_random_order(XfceBackdrop *backdrop,
 
     TRACE("entering");
 
-    backdrop->priv->random_backdrop_order = random_order;
+    if(backdrop->priv->random_backdrop_order != random_order) {
+        backdrop->priv->random_backdrop_order = random_order;
+
+        /* If we have an image list and care about order now, sort the list */
+        if(!random_order && backdrop->priv->image_files) {
+            guint num_items = g_list_length(backdrop->priv->image_files);
+            if(num_items > 1) {
+                backdrop->priv->image_files = sort_image_list(backdrop->priv->image_files, num_items);
+            }
+        }
+    }
 }
 
 gboolean
@@ -1359,47 +1521,18 @@ xfce_backdrop_get_random_order(XfceBackdrop *backdrop)
     return backdrop->priv->random_backdrop_order;
 }
 
-/**
- * xfce_backdrop_set_cache_pixbuf:
- * @backdrop: An #XfceBackdrop.
- * @cache_pixbuf: When TRUE XfceBackdrop will keep a reference to the current
- *                pixbuf until another one is set.
- *
- * XfceBackdrop can be set to keep a reference to the current backdrop and
- * increment the reference count when xfce_backdrop_get_pixbuf is called.
- * Setting cache_pixbuf to FALSE will cause xfce_backdrop_get_pixbuf to
- * return XfceBackdrop's only reference and free any memory allocated. This
- * will use less memory, especially when the backdrop isn't expected to change
- * as in single workspace mode.
- **/
 void
-xfce_backdrop_set_cache_pixbuf(XfceBackdrop *backdrop,
-                               gboolean cache_pixbuf)
+xfce_backdrop_force_cycle(XfceBackdrop *backdrop)
 {
     g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
 
     TRACE("entering");
 
-    if(backdrop->priv->cache_pixbuf == cache_pixbuf) {
-        DBG("No change, cache_pixbuf %s", cache_pixbuf ? "TRUE" : "FALSE");
-        return;
-    }
+    /* force it to update */
+    xfce_backdrop_cycle_backdrop(backdrop);
 
-    backdrop->priv->cache_pixbuf = cache_pixbuf;
-
-    DBG("cache_pixbuf now %s", cache_pixbuf ? "TRUE" : "FALSE");
-
-    /* release any cached pixbuf now to save memory */
-    if(!cache_pixbuf)
-        xfce_backdrop_clear_cached_image(backdrop);
-}
-
-gboolean
-xfce_backdrop_get_cache_pixbuf(XfceBackdrop *backdrop)
-{
-    g_return_val_if_fail(XFCE_IS_BACKDROP(backdrop), FALSE);
-
-    return backdrop->priv->cache_pixbuf;
+    /* Update the timer, if running */
+    xfce_backdrop_timer(backdrop);
 }
 
 /* Generates the background that will either be displayed or will have the
@@ -1454,10 +1587,6 @@ xfce_backdrop_image_data_release(XfceBackdropImageData *image_data)
 
     if(image_data->loader)
         g_object_unref(image_data->loader);
-
-    g_free(image_data);
-    image_data = NULL;
-
 }
 
 /**
@@ -1473,16 +1602,9 @@ xfce_backdrop_get_pixbuf(XfceBackdrop *backdrop)
 {
     TRACE("entering");
 
-    if(backdrop->priv->pix && backdrop->priv->cache_pixbuf) {
+    if(backdrop->priv->pix) {
         /* return a reference so we can cache it */
         return g_object_ref(backdrop->priv->pix);
-    } else if(backdrop->priv->pix) {
-        /* We're not going to cache it so return our reference and set our
-         * pointer to NULL */
-        GdkPixbuf *pix = backdrop->priv->pix;
-        backdrop->priv->pix = NULL;
-
-        return pix;
     }
 
     /* !backdrop->priv->pix, call xfce_backdrop_generate_async */
@@ -1529,7 +1651,7 @@ xfce_backdrop_generate_async(XfceBackdrop *backdrop)
     else
         image_path = DEFAULT_BACKDROP;
 
-    DBG("loading image %s", image_path);
+    XF_DEBUG("loading image %s", image_path);
 
     file = g_file_new_for_path(image_path);
 
@@ -1565,8 +1687,8 @@ xfce_backdrop_loader_size_prepared_cb(GdkPixbufLoader *loader,
     TRACE("entering");
 
     if(backdrop->priv->image_style == XFCE_BACKDROP_IMAGE_INVALID) {
-        g_warning("Invalid image style, setting to XFCE_BACKDROP_IMAGE_STRETCHED");
-        backdrop->priv->image_style = XFCE_BACKDROP_IMAGE_STRETCHED;
+        g_warning("Invalid image style, setting to XFCE_BACKDROP_IMAGE_ZOOMED");
+        backdrop->priv->image_style = XFCE_BACKDROP_IMAGE_ZOOMED;
     }
 
     switch(backdrop->priv->image_style) {
@@ -1635,6 +1757,7 @@ xfce_backdrop_loader_closed_cb(GdkPixbufLoader *loader,
     /* canceled? quit now */
     if(g_cancellable_is_cancelled(image_data->cancellable)) {
         xfce_backdrop_image_data_release(image_data);
+        g_free(image_data);
         return;
     }
 
@@ -1679,13 +1802,14 @@ xfce_backdrop_loader_closed_cb(GdkPixbufLoader *loader,
 
     /* no image and not canceled? return just the canvas */
     if(!image && !g_cancellable_is_cancelled(image_data->cancellable)) {
-        DBG("image failed to load, displaying canvas only");
+        XF_DEBUG("image failed to load, displaying canvas only");
         backdrop->priv->pix = final_image;
 
         g_signal_emit(G_OBJECT(backdrop), backdrop_signals[BACKDROP_READY], 0);
 
         backdrop->priv->image_data = NULL;
         xfce_backdrop_image_data_release(image_data);
+        g_free(image_data);
         return;
     }
 
@@ -1783,6 +1907,7 @@ xfce_backdrop_loader_closed_cb(GdkPixbufLoader *loader,
 
     backdrop->priv->image_data = NULL;
     xfce_backdrop_image_data_release(image_data);
+    g_free(image_data);
 }
 
 static void
@@ -1849,48 +1974,3 @@ xfce_backdrop_file_input_stream_ready_cb(GObject *source_object,
         g_object_unref(source_object);
     }
 }
-
-/* returns TRUE if they have identical settings. */
-gboolean
-xfce_backdrop_compare_backdrops(XfceBackdrop *backdrop_a,
-                                XfceBackdrop *backdrop_b)
-{
-    if(g_strcmp0(backdrop_a->priv->image_path, backdrop_b->priv->image_path) != 0) {
-        DBG("filename different");
-        return FALSE;
-    }
-
-    if(backdrop_a->priv->image_style != backdrop_b->priv->image_style) {
-        DBG("image_style different");
-        return FALSE;
-    }
-
-    if(backdrop_a->priv->color_style != backdrop_b->priv->color_style) {
-        DBG("color_style different");
-        return FALSE;
-    }
-
-    /* Every color style uses color1 except for transparent which does not need
-     * a color check */
-    if(backdrop_a->priv->color_style != XFCE_BACKDROP_COLOR_TRANSPARENT &&
-       !gdk_color_equal(&backdrop_a->priv->color1, &backdrop_b->priv->color1)) {
-        DBG("colors different");
-        return FALSE;
-    }
-
-    /* When the style is set to gradient then we should check color2 as well */
-    if((backdrop_a->priv->color_style == XFCE_BACKDROP_COLOR_HORIZ_GRADIENT ||
-        backdrop_a->priv->color_style == XFCE_BACKDROP_COLOR_VERT_GRADIENT) &&
-       !gdk_color_equal(&backdrop_a->priv->color2, &backdrop_b->priv->color2)) {
-        DBG("colors different");
-        return FALSE;
-    }
-
-    if(backdrop_a->priv->cycle_backdrop != backdrop_b->priv->cycle_backdrop ||
-       backdrop_a->priv->cycle_backdrop == TRUE) {
-        DBG("backdrop cycle different");
-        return FALSE;
-    }
-
-    return TRUE;
-}
